import os
import re
import sys
import logging
import time
import requests
from bs4 import BeautifulSoup

if sys.version_info[:2] <= (2, 7):
    # Python 2
    get_input = raw_input
    str_type = unicode
    import ConfigParser as configparser
else:
    # Python 3
    get_input = input
    str_type = str
    import configparser

class Garc(object):
    """
    Garc allows you retrieve data from the Gab API.
    """

    def __init__(self, user_account=None, user_password=None,
                 connection_errors=0, http_errors=0, profile='main', config=None):
        """
        Create a Garc instance. If account informaton isn't given it will search for them.
        """

        self.user_account = user_account
        self.user_password = user_password
        self.connection_errors = connection_errors
        self.http_errors = http_errors
        self.cookie = None
        self.profile = profile
        self.search_types = ['date']



        if config:
            self.config = config
        else:
            self.config = self.default_config()

        self.check_keys()

    def search(self, q, search_type='date', gabs=-1):
        """
        Pass in a query. Defaults to recent sort by date.
        Defaults to retrieving as many historical gabs as possible.
        """
        # This can be expanded to other Gab search types
        if search_type in self.search_types:
            search_type = search_type
        else:
            search_type = 'date'

        num_gabs = 0
        while True:

            url = "https://gab.ai/api/search?q=%s&sort=%s&before=%s" % (q, search_type, num_gabs)

            resp = self.get(url)

            # We should probably implement some better error catching
            # not simply checking for a 500 to know we've gotten all the gabs possible
            if resp.status_code == 500:
                break
            posts = resp.json()["data"]
            for post in posts:
                yield post
            num_gabs += len(posts)
            if  (num_gabs > gabs and gabs != -1):
                break


    def user(self, q):
        """
        collect user json data
        """
        url = 'https://gab.ai/users/%s' % (q)
        resp = self.get(url)
        yield resp.json()


    def userposts(self, q, gabs=-1, gabs_after='2000-01-01'):
        """
        collect posts from a user feed
        """
        base_url = "https://gab.ai/api/feed/%s" % (q)
        url = base_url
        num_gabs = 0
        while True:
            resp = self.get(url)
            posts = resp.json()["data"]
            if not posts:
                break
            last_published_date = posts[-1]['published_at']

            url = base_url + '?before=%s' % (last_published_date)
            for post in posts:
                yield post
            num_gabs += len(posts)
            if last_published_date < gabs_after:
                break
            if  (num_gabs > gabs and gabs != -1):
                break

    def usercomments(self, q):
        """
        collect comments from a users feed
        """
        base_url = "https://gab.ai/api/feed/%s/comments?includes=post.conversation_parent" % (q)
        url = base_url
        while True:

            resp = self.get(url)
            posts = resp.json()["data"]
            if not posts:
                break
            last_published_date = posts[-1]['published_at']
            url = base_url + '&before=%s' % (last_published_date)
            for post in posts:
                yield post

    def login(self):
        """
        Login to Gab to retrieve needed session token.
        """
        if not (self.user_account and self.user_password):
            raise RuntimeError("MissingAccountInfo")

        if self.cookie:
            logging.info("refreshing login cookie")

        url = "https://gab.ai/auth/login"
        input_token = requests.get('https://gab.ai/auth/login')
        page_info = BeautifulSoup(input_token.content, "html.parser")
        token = page_info.select('input[name=_token]')[0]['value']
        payload = {'username':self.user_account, 'password':self.user_password, '_token':token}

        d = requests.request("POST", url, params=payload, cookies=input_token.cookies)
        laravel_session = re.search('"id_token": "(.+)" }', d.content.decode('utf-8')).group(1)
        self.cookie = {'laravel_session': laravel_session}

    def get(self, url, **kwargs):
        """
        Perform the API requests
        """
        if not self.cookie:
            self.login()

        connection_error_count = kwargs.pop('connection_error_count', 0)
        try:
            logging.info("getting %s %s", url, kwargs)

            r = requests.get(url, cookies=self.cookie)
            # Maybe should implement allow_404 that stops retrying, ala twarcf

            if r.status_code == 404:
                logging.warn("404 from Gab API! trying again")
                time.sleep(1)
                r = self.get(url, **kwargs)
            return r
        except requests.exceptions.ConnectionError as e:
            connection_error_count += 1
            logging.error("caught connection error %s on %s try", e,
                          connection_error_count)
            if (self.connection_errors and
                    connection_error_count == self.connection_errors):
                logging.error("received too many connection errors")
                raise e
            else:
                self.connect()
                kwargs['connection_error_count'] = connection_error_count
                return self.get(url, **kwargs)

    def check_keys(self):
        """
        Get the Gab account info. Order of precedence is command line,
        environment, config file. Return True if all the keys were found
        and False if not.
        """
        env = os.environ.get
        if not self.user_account:
            self.user_account = env('GAB_ACCOUNT')
        if not self.user_password:
            self.user_password = env('GAB_PASSWORD')


        if self.config and not (self.user_account and
                                self.user_password):
            self.load_config()

        return self.user_password and self.user_password

    def load_config(self):
        """
        Attempt to load gab info from config file
        """
        path = self.config
        profile = self.profile
        logging.info("loading %s profile from config %s", profile, path)

        if not path or not os.path.isfile(path):
            return {}

        config = configparser.ConfigParser()
        config.read(self.config)
        data = {}
        for key in ['user_account', 'user_password']:
            try:
                setattr(self, key, config.get(profile, key))
            except configparser.NoSectionError:
                sys.exit("no such profile %s in %s" % (profile, path))
            except configparser.NoOptionError:
                sys.exit("missing %s from profile %s in %s" % (
                    key, profile, path))
        return data

    def save_config(self):
        """
        Save new config file
        """
        if not self.config:
            return
        config = configparser.ConfigParser()
        config.add_section(self.profile)
        config.set(self.profile, 'user_account', self.user_account)
        config.set(self.profile, 'user_password', self.user_password)
        with open(self.config, 'w') as config_file:
            config.write(config_file)

    def input_keys(self):
        """
        Create new config file with account info
        """
        print("\nPlease enter Gab account info.\n")

        def i(name):
            return get_input(name.replace('_', ' ') + ": ")

        self.user_account = i('user_account')
        self.user_password = i('password')
        self.save_config()

    def default_config(self):
        """
        Default config file path
        """
        return os.path.join(os.path.expanduser("~"), ".garc")



from __future__ import print_function

import sys
import json
import signal
import codecs
import logging
import argparse
from garc import __version__
from garc.client import Garc

if sys.version_info[:2] <= (2, 7):
    # Python 2
    get_input = raw_input
    str_type = unicode
else:
    # Python 3
    get_input = input
    str_type = str


commands = [
    'configure',
    'help',
    'sample',
    'search',
    'user',
    'userposts',
    'usercomments'
]


def main():
    parser = get_argparser()
    args = parser.parse_args()

    command = args.command
    query = args.query or ""

    logging.basicConfig(
        filename=args.log,
        level=logging.INFO,
        format="%(asctime)s %(levelname)s %(message)s"
    )

    # catch ctrl-c so users don't see a stack trace
    signal.signal(signal.SIGINT, lambda signal, frame: sys.exit(0))

    if command == "version":
        print("garc v%s" % __version__)
        sys.exit()
    elif command == "help" or not command:
        parser.print_help()
        print("\nPlease use one of the following commands:\n")
        for cmd in commands:
            print(" - %s" % cmd)
        print("\nFor example:\n\n    garc search make america great again")
        sys.exit(1)

    g = Garc(
        user_account=args.user_account,
        user_password=args.user_password,
        connection_errors=args.connection_errors,
        http_errors=args.http_errors,
        config=args.config,
        profile=args.profile)

    # calls that return gabs
    if command == "search":
        things = g.search(
            query,
            search_type=args.search_type,
            gabs=args.number_gabs
        )

    elif command == "configure":
        g.input_keys()
        sys.exit()
    elif command == 'user':
        things = g.user(query)

    elif command == 'userposts':
        things = g.userposts(
            query,
            gabs=args.number_gabs,
            gabs_after=args.gabs_after
        )
    elif command == 'usercomments':
        things = g.usercomments(query)
    else:
        parser.print_help()
        print("\nPlease use one of the following commands:\n")
        for cmd in commands:
            print(" - %s" % cmd)
        print("\nFor example:\n\n    garc search make america great again")
        sys.exit(1)

    # get the output filehandle
    if args.output:
        fh = codecs.open(args.output, 'wb', 'utf8')
    else:
        fh = sys.stdout


    
    for thing in things:
        if 'post' in thing or 'username' in thing:
            # gabs and users
            if args.format == "json":
                print(json.dumps(thing), file=fh)
            logging.info("archived %s", thing['id'])

def get_argparser():
    """
    Get the command line argument parser.
    """

    parser = argparse.ArgumentParser("garc")
    parser.add_argument('command', choices=commands)
    parser.add_argument('query', nargs='?', default=None)
    parser.add_argument("--log", dest="log",
                        default="garc.log", help="log file")
    parser.add_argument("--user_account",
                        default=None, help="Gab account name")
    parser.add_argument("--user_password",
                        default=None, help="Gab account password")
    parser.add_argument('--config',
                        help="Config file containing Gab account info")
    parser.add_argument('--profile', default='main',
                        help="Name of a profile in your configuration file")
    parser.add_argument('--warnings', action='store_true',
                        help="Include warning messages in output")
    parser.add_argument("--connection_errors", type=int, default="0",
                        help="Number of connection errors before giving up")
    parser.add_argument("--http_errors", type=int, default="0",
                        help="Number of http errors before giving up")
    parser.add_argument("--output", action="store", default=None,
                        dest="output", help="write output to file path")
    parser.add_argument("--format", action="store", default="json",
                        dest="format", choices=["json"],
                        help="set output format")
    parser.add_argument("--search_type", action="store", default="date",
                        dest="search_type", choices=["date"],
                        help="set search type")
    parser.add_argument("--number_gabs", action="store", type=int, default=-1,
                        dest="number_gabs",
                        help="approximate number of gabs to return")
    parser.add_argument("--gabs_after", action="store", default="2000-01-01",
                        dest="gabs_after",
                        help="approximate date of earliest gab you wish to collect")


    return parser



import sys
import os

from os.path import join
from setuptools import setup

__version__ = '1.1.5'


with open("README.md") as f:
    long_description = f.read()

if sys.version_info[0] < 3:
    dependencies = open(join('requirements', 'python2.txt')).read().split()
else:
    dependencies = open(join('requirements', 'python3.txt')).read().split()

if __name__ == "__main__":
    setup(
        name='garc',
        version=__version__,
        url='https://github.com/ChrisStevens/garc',
        author='Chris Stevens',
        author_email='chris.stevens@mail.utoronto.ca',
        packages=['garc',],
        description='Archive Gab.ai posts from the command line',
        long_description=long_description,
        long_description_content_type="text/markdown",
        install_requires=dependencies,
        setup_requires=['pytest-runner'],
        tests_require=['pytest'],
        entry_points={'console_scripts': ['garc = garc:main']}
    )